from flask import Flask, render_template, request, jsonify, send_from_directory
from compressing import compress_results_files
from werkzeug.utils import secure_filename
from flask_cors import CORS
from grafana import BESS, FULL, COST, CO2
import numpy as np
import time
import os


app = Flask(__name__)
CORS(app)
app.config["DEBUG"] = True


DOWNLOAD_DIRECTORY = "."
DOWNLOAD_INPUTFILE_DIRECTORY = "./static"

####################################################################################################
########################################## GRAFANA ENDPOINTS #######################################
####################################################################################################
from calendar import timegm
from datetime import datetime
import _strptime  # https://bugs.python.org/issue7980

def convert_to_time_ms(timestamp):
    print(f'timestamp: {timestamp}')
    return 1000 * timegm(datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())

@app.route('/')
def hello():
    time.sleep(5)
    return jsonify({'text':'Hello World!'})

@app.route('/search', methods=['POST'])
def search():
    param = []
    for optimization in ["BESS", "FULL", "COST", "CO2"]:
        for data in ["Overview", "PTER", "PEDS", "PESS", "EESS", "load-shedding"]:
            param.append(f"{data}-{optimization}")

    return jsonify(param)


@app.route('/query', methods=['POST'])
def query():
    req = request.get_json()
    target = req['targets'][0]['target']
    
    #####################################  Optimization ############################################
    if req['targets'][0]['target'] == 'Overview-BESS':
        data = BESS.get_table()
        return jsonify(data)
    
    if req['targets'][0]['target'] == 'Overview-FULL':
        data = FULL.get_table()
        return jsonify(data)
    
    if req['targets'][0]['target'] == 'Overview-COST':
        data = COST.get_table()
        return jsonify(data)
    
    if req['targets'][0]['target'] == 'Overview-CO2':
        data = CO2.get_table()
        return jsonify(data)

    if "BESS" in req['targets'][0]['target']:
        data = BESS.get_timeseries(target.replace("-BESS",""))
        return jsonify(data)
    
    if "FULL" in req['targets'][0]['target']:
        data = FULL.get_timeseries(target.replace("-FULL",""))
        return jsonify(data)

    if "COST" in req['targets'][0]['target']:
        data = COST.get_timeseries(target.replace("-COST",""))
        return jsonify(data)

    if "CO2" in req['targets'][0]['target']:
        data = CO2.get_timeseries(target.replace("-CO2",""))
        return jsonify(data)

####################################################################################################
########################################## ANGULAR ENDPOINTS #######################################
####################################################################################################

@app.route('/downloader', methods = ['GET'])
def download_zip():
    """Download a file."""
    try:
        return send_from_directory(DOWNLOAD_DIRECTORY, 'results.zip', as_attachment=True)
    except FileNotFoundError:
        abort(404)

# End points for the FULL
@app.route('/inputfilefull', methods = ['GET'])
def download_file_full():
    """Download the input data file."""
    try:
        return send_from_directory(
            DOWNLOAD_INPUTFILE_DIRECTORY, 'data_source_optimal_sizing_full.xlsx', as_attachment=True
        )
    except FileNotFoundError:
        abort(404)

@app.route('/uploaderFULL', methods = ['GET', 'POST'])
def upload_file_FULL():
   if request.method == 'POST':
        f = request.files['file']
        f.save(secure_filename(f.filename))
        filename = request.form['filename']
        try:
            os.system(
                'python optimizer/opt_fullsizing/main.py -f ' + secure_filename(filename) + ' -s 1'
            )
            compress_results_files()
            return jsonify({'text':'OK!'})
        except:
            try:
                os.remove(str(f.filename))
            except OSError as e:
                return jsonify({'text':'Error: file not found!', 'filename':str(f.filename)})
            return jsonify({'text':'Error!!!'})
        
# End points for the PV
@app.route('/inputfilepv', methods = ['GET'])
def download_file_pv():
    """Download the input data file."""
    try:
        return send_from_directory(
            DOWNLOAD_INPUTFILE_DIRECTORY, 'data_source_optimal_sizing_PV.xlsx', as_attachment=True
        )
    except FileNotFoundError:
        abort(404)

@app.route('/uploaderPV', methods = ['GET', 'POST'])
def upload_file_PV():
   if request.method == 'POST':
        f = request.files['file']
        f.save(secure_filename(f.filename))
        filename = request.form['filename']
        try:
            os.system(
                'python optimizer/opt_PV/main.py -f ' + secure_filename(filename) + ' -s 1'
            )
            compress_results_files()
            return jsonify({'text':'OK!'})
        except:
            try:
                os.remove(str(f.filename))
            except OSError as e:
                return jsonify({'text':'Error: file not found!', 'filename':str(f.filename)})
            return jsonify({'text':'Error!!!'})
        
# End points for the CO2
@app.route('/inputfileco2', methods = ['GET'])
def download_file_co2():
    """Download the input data file."""
    try:
        return send_from_directory(
            DOWNLOAD_INPUTFILE_DIRECTORY, 'data_source_optimal_sizing_CO2.xlsx', as_attachment=True
        )
    except FileNotFoundError:
        abort(404)

@app.route('/uploaderCO2', methods = ['GET', 'POST'])
def upload_file_CO2():
   if request.method == 'POST':
        f = request.files['file']
        f.save(secure_filename(f.filename))
        filename = request.form['filename']
        try:
            os.system(
                'python optimizer/opt_CO2/main.py -f ' + secure_filename(filename) + ' -s 1'
            )
            compress_results_files()
            return jsonify({'text':'OK!'})
        except:
            try:
                os.remove(str(f.filename))
            except OSError as e:
                return jsonify({'text':'Error: file not found!', 'filename':str(f.filename)})
            return jsonify({'text':'Error!!!'})
        
# End points for the BESS
@app.route('/inputfilebess', methods = ['GET'])
def download_file_bess():
    """Download the input data file."""
    try:
        return send_from_directory(
            DOWNLOAD_INPUTFILE_DIRECTORY, 'data_source_optimal_sizing_BESS.xlsx', as_attachment=True
        )
    except FileNotFoundError:
        abort(404)


@app.route('/uploaderBESS', methods = ['GET', 'POST'])
def upload_file_BESS():
   print(request.method)
   if request.method == 'POST':
        f = request.files['file']
        f.save(secure_filename(f.filename))
        filename = request.form['filename']
        try:
            os.system('python optimizer/opt_BESS/main.py -f ' + secure_filename(filename) + ' -s 1 -T 100 -P 500')
            compress_results_files()
            return jsonify({'text':'OK!'})
        except:
            try:
                os.remove(str(f.filename))
            except OSError as e:
                return jsonify({'text':'Error: file not found!', 'filename':str(f.filename)})
            return jsonify({'text':'Error!!!'})


app.run(host='0.0.0.0')