from datetime import datetime, timedelta
from pymongo import MongoClient
import os

MONGOPATH = os.getenv("MONGODB_PATH", "mongodb://mongodb:27017")

def call():
    client = MongoClient(MONGOPATH, connect=False)
    db = client["CO2"]
    return db

def last_col():
    db = call()
    names = db.list_collection_names()
    last = datetime(2023, 1, 1, 0, 0, 0)
    col = None
    if len(names) > 0:
        for date in names:
            start = datetime.fromisoformat(date)
            if start > last:
                last = start

        col = db[last.isoformat()]
    return col

def get_table():
    col = last_col()
    if col is None:
        return
    else:
        result = []
        values = list(col.find({}, {"_id": 0}))[0]
        for v in values:
            if type(values[v]) != type([]):
                result.append([v, str(values[v])])
            

        data = [
        {   
            "columns":[
                {"text":"Optimization Parameter","type":"string"},
                {"text":"Optimization Result","type":"string"}
                ],
            "rows":result,
            "type":"table"
        }
        ]
        return data

def get_timeseries(par):
    col = last_col()
    data = []
    if col is None:
        return
    else:
        values = list(col.find({}, {"_id": 0}))[0]
        for s in values['scenario']:    
            for c in values['contingency']:
                series = {"target": c, "datapoints": []}
                dt = datetime.now()
                for index, t in enumerate(values['time-step']):
                    date = datetime(dt.year, dt.month, dt.day, 0, 0, 0) + timedelta(minutes = 60*float(t))
                    series["datapoints"].append(
                        [
                            values[f'{par}-s{s}-c{c}'][index],
                            datetime.timestamp(date)*1000
                        ]
                    )
                data.append(series)

    return data