from pymongo import MongoClient
from datetime import datetime
import os

MONGOPATH = os.getenv("MONGODB_PATH", "mongodb://mongodb:27017")

def call():
    client = MongoClient(MONGOPATH, connect=False)
    db = client["BESS"]
    return db

def storage(data, results, List_TSC):
    db = call()
    now = datetime.now().isoformat()[:-7]
    col = db[now]

    if results.Status in ["Infeasible", "Not Solved"]:
        save = {
            "Timestamp": now,
            "status": results.Status,
            "Objective-Function" :0 ,
            "Investment-cost [R$]": 0,
            "Operation-cost [R$]": 0,
            "Thermal-generation-cost [R$]": 0,
            "Load-shedding-cost [R$]": 0,
            "Thermal-generation-size [kW]": 0,
            "PV-generation-size [kW]": 0,
            "ESS-power-converter [kW]": 0,
            "ESS-energy-capacity [kWh]": 0,
        }
    else:
        save = {
            "Timestamp": now,
            "status": results.Status,
            "Objective-Function": round(results.ObjectiveFunctionValue,0),
            "Investment-cost [R$]": round(sum([data.cESS[b] * (0.5 * results.MxEESS_beta[b] + 0.5 * results.MxPESS_beta[b]) for b in data.B]),0),
            "Operation-cost [R$]": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cEDS[t] * data.deltat * results.PEDS[t][s][c] for (t,s,c) in List_TSC]),0),
            "Thermal-generation-cost [R$]": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cOTer[t] * data.deltat * results.PTER[t][s][c] for (t,s,c) in List_TSC]),0),
            "Load-shedding-cost [R$]": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.M * data.fD[t][s] * data.MxPD * results.xD[t][s][c] * data.deltat for (t,s,c) in List_TSC]),0),
            "Thermal-generation-size [kW]": round(results.MxPTer,0),
            "PV-generation-size [kW]": round(results.MxPPV,0),
            "ESS-power-converter [kW]": round(results.MxPESS,0),
            "ESS-energy-capacity [kWh]": round(results.MxEESS,0),
        }
    
    contingency = []
    for c in data.C:
        if results.Status == 'Optimal':
            contingency.append(str(int(c)*data.deltat).zfill(1))
    save.update({"contingency": contingency})

    if results.Status == 'Optimal':
        for a in data.A:
            if results.alpha[a] != 0: 
                save.update({'PV-technology': str(data.PV_tech_name[a])})

        for b in data.B:
            if results.beta[b] != 0:
                save.update({'ESS-technology': str(data.ESS_tech_name[b])}) 
    else:
        save.update({'PV-technology': "None"})
        save.update({'ESS-technology': "None"})
    
    Timestep = []
    if results.Status == 'Optimal':
        for t in data.T:
            Timestep.append(int(t) * data.deltat)
    save.update({'time-step': Timestep})

    # SAVE PTER data
    for c in data.C:
        PTER = []
        if results.Status == 'Optimal':
            for t in data.T:
                PTER.append(round(results.PTER[t]['5'][c],2))
        save.update({f'PTER-{str(int(c)*data.deltat).zfill(1)}': PTER})
        
    # SAVE PEDS DATA
    for c in data.C:
        PEDS = []
        if results.Status == 'Optimal':
            for t in data.T:
                PEDS.append(round(results.PEDS[t]['5'][c],2))
        save.update({f'PEDS-{str(int(c)*data.deltat).zfill(1)}': PEDS})

    # SAVE PESS DATA
    for c in data.C:
        PESS = []
        if results.Status == 'Optimal':
            for t in data.T:
                PESS.append(round(results.PESS_in[t]['5'] - results.PESS_out[t]['5'],2))
        save.update({f'PESS-{str(int(c)*data.deltat).zfill(1)}': PESS})

    # SAVE EESS DATA 
    for c in data.C:
        EESS = []
        if results.Status == 'Optimal':
            for t in data.T:
                EESS.append(round(results.EESS[t]['5'],2))
        save.update({f'EESS-{str(int(c)*data.deltat).zfill(1)}': EESS})

    # Apresenta os resultados do corte de carga
    for c in data.C:
        LoadShedding = []
        if results.Status == 'Optimal':
            for t in data.T:
                LoadShedding.append(round(data.fD[t]['5'] * data.MxPD * results.xD[t]['5'][c],2))
        save.update({f"load-shedding-{str(int(c)*data.deltat).zfill(1)}": LoadShedding})

    # Apresenta os resultados da geracao fotovoltaica
    for c in data.C:
        PVgen = []
        if results.Status == 'Optimal':
            for t in data.T:
                PVgen.append(round(sum([data.fPV[t]['5'] * data.eta[a] * results.MxPPV_alpha_xPV[a][t]['5'][c] for a in data.A]),2))
        save.update({f'PVgen-{str(int(c)*data.deltat).zfill(1)}': PVgen})

    col.insert_one(save)

    return


