from pymongo import MongoClient
from datetime import datetime
import os

MONGOPATH = os.getenv("MONGODB_PATH", "mongodb://mongodb:27017")

def call(name):
    client = MongoClient(MONGOPATH, connect=False)
    db = client[name]
    return db

def storage(data, results, List_TSC, name):
    db = call(name)
    now = datetime.now().isoformat()[:-7]
    col = db[now]
    
    if results.Status in ["Infeasible", "Not Solved"]:
        save = {
        "Timestamp": now,
        "status": results.Status,
        "Optimization type": name,
        "Total cost (CAPEX + OPEX) [R$]": 0,
        "CO2 emission (All scenarios) [kg]": 0,
        "Investment-cost [R$]": 0,
        "Operation-cost [R$]": 0,
	    "Thermal-generation-cost [R$]": 0,
	    "Load-shedding-cost [R$]": 0,
        "Thermal-generation-size [kW]": 0,
        "PV-generation-size [kW]": 0,
        "ESS-power-converter [kW]": 0,
        "ESS-energy-capacity [kWh]": 0,
    }

    else:
        save = {
            "Timestamp": now,
            "status": results.Status,
            "Optimization type": name,
            "Total cost (CAPEX + OPEX) [R$]": round(results.varDic['COST']),
            "CO2 emission (All scenarios) [kg]": round(results.varDic['CO2']),
            "Investment-cost": round(sum([data.cESS[b] * (0.5 * results.MxEESS_beta[b] + 0.5 * results.MxPESS_beta[b]) for b in data.B]),0),
            "Operation-cost": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cEDS[t] * data.deltat * results.PEDS[t][s][c] for (t,s,c) in List_TSC]),0),
            "Thermal-generation-cost": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cOTer[t] * data.deltat * results.PTER[t][s][c] for (t,s,c) in List_TSC]),0),
            "Load-shedding-cost": round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.M * data.fD[t][s] * data.MxPD * results.xD[t][s][c] * data.deltat for (t,s,c) in List_TSC]),0),
            "Thermal-generation-size": round(results.MxPTer,0),
            "PV-generation-size": round(results.MxPPV,0),
            "ESS-power-converter": round(results.MxPESS,0),
            "ESS-energy-capacity": round(results.MxEESS,0),
        }
    contingency = []
    for c in data.C:
        if results.Status == 'Optimal':
            contingency.append(str(int(c)*data.deltat).zfill(1))
    save.update({"contingency": contingency})

    scenario = []
    for s in data.S:
        if results.Status == 'Optimal':
            scenario.append(s)
    save.update({"scenario": scenario})

    if results.Status == 'Optimal':
        for a in data.A:
            if results.alpha[a] != 0: 
                save.update({'PV-technology': str(data.PV_tech_name[a])})
        for b in data.B:
            if results.beta[b] != 0:
                save.update({'ESS-technology': str(data.ESS_tech_name[b])})
    else:
        save.update({'PV-technology': "None"})
        save.update({'ESS-technology': "None"})

    Timestep = []
    if results.Status == 'Optimal':
        for t in data.T:
            Timestep.append(int(t) * data.deltat)

    save.update({'time-step': Timestep})

    # SAVE PTER data
    for c in data.C:
        for s in data.S:
            PTER = []
            if results.Status == 'Optimal':
                for t in data.T:
                    PTER.append(round(results.PTER[t][s][c],2))
            save.update({f'PTER-s{s}-c{str(int(c)*data.deltat).zfill(1)}': PTER})
        

    # SAVE PEDS DATA
    for c in data.C:
        for s in data.S:
            PEDS = []
            if results.Status == 'Optimal':
                for t in data.T:
                    PEDS.append(round(results.PEDS[t][s][c],2))
            save.update({f'PEDS-s{s}-c{str(int(c)*data.deltat).zfill(1)}': PEDS})

    # SAVE PESS DATA
    for c in data.C:
        for s in data.S:
            PESS = []
            if results.Status == 'Optimal':
                for t in data.T:
                    PESS.append(round(results.PESS_in[t][s][c] - results.PESS_out[t][s][c],2))
            save.update({f'PESS-s{s}-c{str(int(c)*data.deltat).zfill(1)}': PESS})

    # SAVE EESS DATA 
    for c in data.C:
        for s in data.S:
            EESS = []
            if results.Status == 'Optimal':
                for t in data.T:
                    EESS.append(round(results.EESS[t][s][c],2))
            save.update({f'EESS-s{s}-c{str(int(c)*data.deltat).zfill(1)}': EESS})

    # Apresenta os resultados do corte de carga
    for c in data.C:
        for s in data.S:
            LoadShedding = []
            if results.Status == 'Optimal':
                for t in data.T:
                    LoadShedding.append(round(data.fD[t]['5'] * data.MxPD * results.xD[t][s][c],2))
            save.update({f"load-shedding-s{s}-c{str(int(c)*data.deltat).zfill(1)}": LoadShedding})

    # Apresenta os resultados da geracao fotovoltaica
    for c in data.C:
        for s in data.S:
            PVgen = []
            if results.Status == 'Optimal':
                for t in data.T:
                    PVgen.append(round(sum([data.fPV[t][s] * data.eta[a] * results.MxPPV_alpha_xPV[a][t][s][c] for a in data.A]),2))
            save.update({f'PVgen-s{s}-c{str(int(c)*data.deltat).zfill(1)}': PVgen})

    col.insert_one(save)
    return
    # datetime.fromisoformat(now)
