""""
==========================Processing source data excel file=========================
"""
import pandas as pd
import tkinter.messagebox

class InputData:
	def __init__(self, path): # Passing arguments given to the class instantiation operator (_init_) to the class 
		self.path = path # Creating the attribute path
		self.sheet1 = "PV_technologies"
		self.sheet2 = "ESS_technologies"
		self.sheet3 = "electricity_costs"
		self.sheet4 = "thermal_costs"
		self.sheet5 = "contingencies"
		self.sheet6 = "PV_production"
		self.sheet7 = "demand"
		self.sheet8 = "individual_parameters"
		
		# Creating sets as list in Python
		self.T = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30', '31', '32', '33', '34', '35', '36', '37', '38', '39', '40', '41', '42', '43', '44', '45', '46', '47', '48']
		self.Y = ['1', '2', '3', '4', '5']
		self.A = ['1', '2', '3', '4']
		self.B = ['1', '2', '3', '4']

	# Function to verify if the excel file selected by the user is the correct excel file to be read by the python program
	def ValidatingInputFile1(self, args_on):
		saved_args = locals()
		try:
			df0 = pd.read_excel(self.path,self.sheet1)
		except Exception as e:
			if args_on == 1:
				tkinter.messagebox.showinfo("Error!", "The selected  input data file is not correct. Please select the correct file.")
				return (0,"ERROR")
			else:
				return (0,"ERROR")
		return (df0,"OK",saved_args)


	# Function to validate if the open excel file is the correct file! 
	# The correct file should contain hidden numbers in row 3 of the first sheet which sum is equal to 10!
	def ValidatingInputFile2(self, dataframe):
		saved_args = locals()
		check1 = dataframe.iloc[1]['List of available PV tecnhologies']
		check2 = dataframe.iloc[1]['Unnamed: 1']
		check3 = dataframe.iloc[1]['Unnamed: 2']
		check4 = dataframe.iloc[1]['Unnamed: 3']
		checking_criteria = check1 + check2 + check3 + check4

		if checking_criteria == 10:
			pass
		else:
			tkinter.messagebox.showinfo("Error!", "The selected  input data file is not correct. Please select the correct file.")
			return("ERROR", saved_args)
		return("OK", saved_args) 
	
	# Function to read an specific excel sheet and save the info in a dataframe
	def GettingDataFrame(self,sheet,skiprows):
		saved_args = locals()
		dataframe = pd.read_excel(self.path,sheet,skiprows)
		return(dataframe, saved_args)
		
	# Function to validate if there are any missing values
	def CheckingMissingValues(self,dataframe,sheet):
		saved_args = locals()
		missing_values = dataframe.isnull().values.any()

		if sheet == self.sheet5:
			# Total number of missing values
			if missing_values == True:
				# Calculate how many missing values are in the dataframe
				total_mv = dataframe.isnull().sum().sum()
				
				# Counting the number of data in the sheet (rows and columns )
				columns_data = sum(dataframe.count(axis='columns')) # Counting data in all columns including time and hour without considering head titles 
				rows_number = len(dataframe.index)

				# Defining the number of cells that must be filled in this sheet 
				filled_values =  columns_data - rows_number*2 # Multiplied by 2 because info in columns "Time" and "Hour" should not count

				if filled_values == 0: 

					# Calculating probability by default (equal probability for all the scenarios)
					eqprob = 1/total_mv
					
					# Replacing missing values with a default number
					dataframe.fillna(eqprob, inplace=True)

					# Verifying that the sum of all data in df4 is equal to 1
					sum_data = dataframe.sum(axis=0).sum() - dataframe['Time'].sum() # Resting the values in the columns "Time"

					return ("PROB_CALCULATED_OK", sum_data, saved_args)
				else:
					text = "There are (%s) missing values in the data source file - sheet:" + " " + sheet + ". Please fill each cell in the sheet with a proper value." 
					tkinter.messagebox.showinfo("warning", text % total_mv)
					return ("Warning", total_mv, saved_args)
			else: 
				# Verifying that the sum of all data in df is equal to 1
				sum_data = dataframe.sum(axis=0).sum() - dataframe['Time'].sum() # Resting the values in the columns "Time"
				if sum_data >= 0.9999 and sum_data <= 1.0001:
					return ("PROB_OK", sum_data, saved_args)
				else:
					text = "The sum of the probabilities in the data source file - sheet: " + " " + sheet + " is not equal to 1. Please fill each cell in the sheet with the proper values. In case this information is not available, please leave all the cells empty and execute this program again!"
					tkinter.messagebox.showinfo("warning", text )
					return ("PROB_SUM_Warning", sum_data, saved_args)
		else: 
			if missing_values == True:
				total_mv = dataframe.isnull().sum().sum()
				text = "There are (%s) missing values in the data source file - sheet:" + " " + sheet + ". Please fill each cell in the sheet with a proper value." 
				tkinter.messagebox.showinfo("warning", text % total_mv)
				return ("Warning", total_mv, saved_args)
			else: 
				return ("OK", 0,saved_args)


	def ProcessingInfoDataFramePV(self, dataframe):
		# Another way to transform dataframe columns into strings
		dataframe['Type'] = dataframe['Type'].apply(str)
		
		# Creating directories for the efficiency and the costs of the PV technologies
		self.eta = dataframe.set_index('Type')['Efficiency_[%]'].to_dict()
		self.cPV = dataframe.set_index('Type')['Cost_[R$/kW]'].to_dict()
		self.PV_tech_name = dataframe.set_index('Type')['Name'].to_dict()
		self.PV_CO2 = dataframe.set_index('Type')['C02_[kg/kWh]'].to_dict()

		return (self.eta, self.cPV, self.PV_tech_name, self.PV_CO2)

	def ProcessingInfoDataFrameESS(self, dataframe):
		# Another way to transform dataframe columns into strings
		dataframe['Type'] = dataframe['Type'].apply(str)

		# Creating directories for the efficiency and the costs of the PV technologies
		self.eta_ESS = dataframe.set_index('Type')['Efficiency_[%]'].to_dict()
		self.ramp = dataframe.set_index('Type')['DoD_[%]'].to_dict()
		self.cESS = dataframe.set_index('Type')['Cost_[R$/kW]'].to_dict()
		self.ESS_tech_name = dataframe.set_index('Type')['Name'].to_dict()
		self.ESS_CO2 = dataframe.set_index('Type')['C02_[kg/kWh]'].to_dict()
		self.ESS_Ncycles = dataframe.set_index('Type')['Ncycles'].to_dict()
	
		return (self.eta_ESS, self.ramp, self.cESS, self.ESS_tech_name, self.ESS_CO2, self.ESS_Ncycles)

	def ProcessingInfoDataFrameElectricityCostsSubstation(self, dataframe):
		# Transforming dataframe columns into strings
		dataframe['Time'] = dataframe['Time'].apply(str)

		# Creating directory for the electricity costs 
		self.cEDS = dataframe.set_index('Time')['Energy_price'].to_dict()
		return (self.cEDS)

	def ProcessingInfoDataFrameElectricityCostsThermal(self, dataframe):
		# Transforming dataframe columns into strings
		dataframe['Time'] = dataframe['Time'].apply(str)

		# Creating directory for the electricity costs 
		self.cOTer = dataframe.set_index('Time')['Energy_price'].to_dict()
		return (self.cOTer)

	def ProcessingInfoDataFrameContingencies(self, dataframe):
		# To eliminate or drop columns not required 
		dataframe = dataframe.drop(['Hour'],axis=1)

		# Transposing this specific dataframe according to the way the dictionary is in the data file
		# and defining the indexes using the data from column Time 
		dataframe = dataframe.set_index('Time').T

		# To transform dataframe columns into strings
		dataframe.columns = dataframe.columns.astype(str)
		# print(df4.columns.map(type))

		# Transforming dataframe index (rows) into Strings
		dataframe.index = dataframe.index.map(str)

		# Use the orient=index parameter to have the index as dictionary keys
		self.p = dataframe.to_dict(orient="index",)
		return (self.p)
		
	def  ProcessingInfoDataFramePVproduction(self, dataframe):
		# To transform dataframe columns into strings
		dataframe.columns = dataframe.columns.astype(str)

		# To eliminate or drop columns not required 
		dataframe = dataframe.drop(['Hour'],axis=1)

		# To set the Time column as the dataframe index. 
		dataframe.set_index("Time", drop=True, inplace=True)

		# Transforming dataframe indexes (rows) into Strings
		dataframe.index = dataframe.index.map(str)

		# Use the orient=index parameter to have the index as dictionary keys
		self.fPV = dataframe.to_dict(orient="index",)
		return (self.fPV)

	def  ProcessingInfoDataFrameDemand(self, dataframe):
		# To transform dataframe columns into strings
		dataframe.columns = dataframe.columns.astype(str)

		# To eliminate or drop columns not required 
		dataframe = dataframe.drop(['Hour'],axis=1)

		# To set the Time column as the dataframe index. 
		dataframe.set_index("Time", drop=True, inplace=True)

		# Transforming dataframe indexes (rows) into Strings
		dataframe.index = dataframe.index.map(str)

		# Use the orient=index parameter to have the index as dictionary keys
		self.fD = dataframe.to_dict(orient="index",)
		return (self.fD)
		
	def ProcessingInfoDataFrameIndividualParameters(self, dataframe):
		self.MxPD           = dataframe.iloc[0]['Value']
		self.MxPEDS         = dataframe.iloc[1]['Value']
		self.cITer          = dataframe.iloc[2]['Value']
		self.M              = dataframe.iloc[3]['Value']
		self.d              = dataframe.iloc[4]['Value']
		self.timeOp         = dataframe.iloc[5]['Value']
		self.mu             = dataframe.iloc[6]['Value']
		self.beta           = dataframe.iloc[7]['Value']
		self.EESS0          = dataframe.iloc[8]['Value']
		self.D              = dataframe.iloc[9]['Value']
		self.deltat         = dataframe.iloc[10]['Value']
		self.r              = dataframe.iloc[11]['Value']
		self.limit_MxPTer   = dataframe.iloc[12]['Value']
		self.limit_MxPPV    = dataframe.iloc[13]['Value']
		self.limit_MxPESS   = dataframe.iloc[14]['Value']
		self.limit_MxEESS   = dataframe.iloc[15]['Value']
		self.INV_LowerLimit = dataframe.iloc[16]['Value']
		self.INV_UpperLimit = dataframe.iloc[17]['Value']
		self.EDS_CO2 		= dataframe.iloc[18]['Value']
		self.TG_CO2			= dataframe.iloc[19]['Value']
		self.INV_inc		= dataframe.iloc[20]['Value']
		self.OPE_inc		= dataframe.iloc[21]['Value']
		return (self.MxPD, self.MxPEDS, self.cITer, self.M, self.d, self.timeOp, self.mu, self.beta, self.EESS0, self.D, self.deltat, self.r, self.limit_MxPTer, self.limit_MxPPV, self.limit_MxPESS, self.limit_MxEESS, self.INV_LowerLimit, self.INV_UpperLimit, self.EDS_CO2, self.TG_CO2)


	def ProcessingInfoTypeofSolution(self, solution_type):
		if solution_type == 1:
			self.S = ['5']
			self.C = ['0','6','18', '30', '42']
			return ("Type_1", self.S, self.C)
		else:
			self.S = ['1', '2', '3', '4', '5', '6', '7', '8', '9']
			self.C = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30', '31', '32', '33', '34', '35', '36', '37', '38', '39', '40', '41', '42', '43', '44', '45', '46', '47', '48']
			return ("Type_2", self.S, self.C)
