""""
=============Software for sizing ESSs (battery/converter) in Microgrids (MERGE - E64)=====================
"""
import os, sys
import tkinter as tk
from tkinter import filedialog
from tools.input_data_processing import InputData
from tools.mathematical_model_construction import MathematicalModel 
from tools.summary_results_construction import SummaryResults
from tools.plotting_results_construction import PlottingResults
from init import InputArguments
import database


#####################################################################
#===== Setting automatic or manual input parameters selections = ====
#####################################################################
filename, solution_type, ThermalSize, PVSize, args_on = InputArguments.InputArgs(__file__) 
print(filename)
print(args_on)
# sys.exit()

""""
==========================Processing source data excel file=========================
"""
#####################################################################
#==== Selecting and opening the excel file from a window browser ====
#####################################################################

if args_on == 1:
	print("\n************* Optimal Sizing of ESSs in Microgrids  *************\n")
	print("\nPlease select input data file...", flush=True)
	# Saving the path of the excel file allowing only excel type files! 
	root = tk.Tk()
	root.withdraw()
	root.filename = filedialog.askopenfilename(initialdir=os.getcwd(), title="Select file",
										filetypes=[("Excel Files", "*.xlsx")])
	print(root.filename)
	print(type(root.filename))
	data = InputData(root.filename) # Criando um objeto da classe InputData
else:
	filename = os.path.abspath(filename)
	data = InputData(filename) # Criando um objeto da classe InputData


# Validating the selection of the correct Input Data File
dataframe, message_validation = data.ValidatingInputFile1(args_on)

if message_validation == "OK":
	pass
else:
	print("Error!", "The selected  input data file is not correct. Please select the correct file.")
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada


print("\n************* START: Reading data input file  *************\n")

# Validating that the selected Input Data File has not been altered
message_validation = data.ValidatingInputFile2(dataframe)
if message_validation == "OK":
	print("1. The correct input data file has been uploaded!", flush=True)
else:
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

#####################################################################
#================= PROCESSING PV TECHNOLOGIES (df) =================
#####################################################################
dataframe = data.GettingDataFrame(data.sheet1,3)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet1)
if message_validation == "OK":
	print("2. Information related to " + data.sheet1 + " complete!", flush=True)
else:
	print("2. Warning! Information related to " + data.sheet1 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to PV technologies
data.ProcessingInfoDataFramePV(dataframe)

#####################################################################
#================= PROCESSING ESS TECHNOLOGIES (df) =================
#####################################################################
dataframe = data.GettingDataFrame(data.sheet2,3)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet2)
if message_validation == "OK":
	print("3. Information related to " + data.sheet2 + " complete!", flush=True)
else:
	print("3. Warning! Information related to " + data.sheet2 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to ESS technologies
data.ProcessingInfoDataFrameESS(dataframe)

#####################################################################
#=======PROCESSING ELECTRICY COSTS FROM SUBSTATION DATA(df) ========
#####################################################################
dataframe = data.GettingDataFrame(data.sheet3,3)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet3)
if message_validation == "OK":
	print("4. Information related to " + data.sheet3 + " complete!", flush=True)
else:
	print("4. Warning! Information related to " + data.sheet3 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to electricity costs from the main grid
data.ProcessingInfoDataFrameElectricityCostsSubstation(dataframe)

######################################################################
#====PROCESSING ELECTRICY COSTS FROM THERMAL GENERATORS DATA (df3) ===
######################################################################
dataframe = data.GettingDataFrame(data.sheet4,3)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet4)
if message_validation == "OK":
	print("5. Information related to " + data.sheet4 + " complete!", flush=True)
else:
	print("5. Warning! Information related to " + data.sheet4 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to electricity costs from thermal generation units
data.ProcessingInfoDataFrameElectricityCostsThermal(dataframe)

######################################################################
#=============== PROCESSING CONTINGENCIES DATA(df) ==================
######################################################################
dataframe = data.GettingDataFrame(data.sheet5,6)

# Checking if there are missing values in the dataframe
message_validation, out = data.CheckingMissingValues(dataframe,data.sheet5)
if message_validation == "PROB_CALCULATED_OK":
	print("6. The user did not provide information on the probability of contingencies.", flush=True)
	print("6.1 Default values for contingencies have been calculated and added.", flush=True)
	print("6.2 The sum of the probabilities is equal to %.2f: " %out, flush=True)
elif message_validation == "warning":
	print("6. Warning! Information related to " + data.sheet5 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada
elif message_validation == "PROB_OK":
	print("6. The sum of the probabilities of contingencies is equal to %.2f: " %out, flush=True)
	print("6.1 Information related to contingencies right and complete!", flush=True)
elif message_validation == "PROB_SUM_Warning":
	print("6. Warning! The sum of the probabilities in the data source file - sheet: contingencies is not equal to 1. \
	Please fill each cell in the sheet with the proper values. \
	In case this information is not available, please leave all the cells empty and execute this program again!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada
elif message_validation == "Warning":
	print("6. Warning! Information related to " + data.sheet5 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to contingencies
data.ProcessingInfoDataFrameContingencies(dataframe)

######################################################################
#=============== PROCESSING PV PRODUCTION DATA (df) ==================
######################################################################
dataframe = data.GettingDataFrame(data.sheet6,5)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet6)
if message_validation == "OK":
	print("7. Information related to " + data.sheet6 + " complete!", flush=True)
else:
	print("7. Warning! Information related to " + data.sheet6 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to PV production scenarios
data.ProcessingInfoDataFramePVproduction(dataframe)

######################################################################
#==================== PROCESSING DEMAND DATA(df) ====================
######################################################################
dataframe = data.GettingDataFrame(data.sheet7,5)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet7)
if message_validation == "OK":
	print("8. Information related to " + data.sheet7 + " complete!", flush=True)
else:
	print("8. Warning! Information related to " + data.sheet7 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to demand scenarios
data.ProcessingInfoDataFrameDemand(dataframe)

######################################################################
#========== PROCESSING INDIVIDUAL PARAMETERS DATA (df) ==============
######################################################################
dataframe = data.GettingDataFrame(data.sheet8,3)

# Checking if there are missing values in the dataframe
message_validation, _ = data.CheckingMissingValues(dataframe,data.sheet8)
if message_validation == "OK":
	print("9. Information related to " + data.sheet8 + " complete!", flush=True)
else:
	print("9. Warning! Information related to " + data.sheet8 + " incomplete!", flush=True)
	sys.exit() # Poner algo para que regrese al inicio y pida de nuevo el archivo de entrada

# Processing the data related to individual parameters
data.ProcessingInfoDataFrameIndividualParameters(dataframe)
 
print("\n************* END: Reading data input file  *************\n", flush=True)

######################################################################
#=================== SELECTING TYPE OF SOLUTION ======================
######################################################################

if args_on == 1:
	print("\n************* START: Selecting type of solution  *************\n", flush=True)
	print("What type of solution do you prefer?:", flush=True)
	print("\nPlease enter '1' to obtain a rapid solution (less than 30 min in a PC Intel (R) Core(TM) i5-323M RAM 6GB):", flush=True)
	print("Please enter '2' to obtain a more detailed solution but it will take much longer:", flush=True)

	solution_type = input()
	if solution_type.isnumeric() == True: 
		solution_type = int(solution_type)

	while solution_type != 1 and solution_type != 2:
		print("Value is not a valid number, please choose option 1 or option 2:")
		solution_type = input()
		if solution_type.isnumeric() == True: 
			solution_type = int(solution_type)
		else:
			print("Value is not a valid number, try again:")
			solution_type = input()
			if solution_type.isnumeric() == True: 
				solution_type = int(solution_type)

	solution_type = int(solution_type)
else:
	solution_type = solution_type

# Defining the values for the lists S and C according to the selected type of solution
message_validation, *_ = data.ProcessingInfoTypeofSolution(solution_type)
if message_validation == "Type_1":
	print("\nType of solution selected: " + "\t" + str(round(solution_type,0)) + " - Rapid ")
else:
	print("\nType of solution selected: " + "\t" + str(round(solution_type,0)) + " - Detailed solution")

print("\n************* END: Selecting type of solution  *************\n", flush=True)

######################################################################
# ======================== CONSOLE INPUT DATA ========================
######################################################################

if args_on == 1:
	print("\nPlease enter the size of the Thermal Generation Unit (TGU) in kW:", flush=True)
	ThermalSize = input()
	flag = 0
	while flag == 0:
		if ThermalSize.isnumeric() == True and float(ThermalSize) <= data.limit_MxPTer:
			ThermalSize = float(ThermalSize)
			flag = 1
		else: 
			print("Value is not a valid number, try again:")
			ThermalSize = input()

	print("\nTGU: " + "\t" + str(round(ThermalSize,0)) + "\t" + "kW")

	print("\nPlease enter the nominal power of the PV system (PV) in kW:", flush=True)
	PVSize = input()
	flag = 0
	while flag == 0:
		if PVSize.isnumeric() == True and float(PVSize) <= data.limit_MxPPV:
			PVSize = float(PVSize)
			flag = 1
		else: 
			print("Value is not a valid number, try again:")
			PVSize = input()

	print("\nPV system Power: " + "\t" + str(round(PVSize,0)) + "\t" + "kW")
else:
	ThermalSize = ThermalSize
	PVSize = PVSize
	print("\nTGU: " + "\t" + str(round(ThermalSize,0)) + "\t" + "kW")
	print("\nPV system Power: " + "\t" + str(round(PVSize,0)) + "\t" + "kW")

# Saving console input data as attributes of InputData class
data.ProcessingConsoleInputData(ThermalSize, PVSize)

""""
========================== Constructing Mathematical Model =========================
"""
print("\n************* START: Formulating mathematical model  *************\n", flush=True)

model = MathematicalModel(data) # Criando um objeto da classe MathematicalModel

# Creating necessary list for the model construction
List_TSC, List_TS, List_TSB, List_ATSC = model.ConstructionOfLists()

# Formulating the problem 
prob, MxPTer, MxPPV, MxPESS, MxEESS, alpha, beta = model.ProblemFormulation(List_TSC, List_TS, List_TSB, List_ATSC)

# Writing the problem in a file .lp
model.WritingProblemFile(prob, "Optimal_Microgrid_Size")

## Defining an specific solution for the problem
#prob = model.TestingEspecificSolution(prob, 304, 375, 4, 1, MxPESS, MxEESS, alpha, beta)

print("\n************* END: Formulating mathematical model  *************\n", flush=True)

######################################################################
#================ SOLVING MATHEMATICAL MODEL ====================
######################################################################

print("\n************* START: Solving mathematical model  *************\n", flush=True)

results = model # Giving attributes of object model to new object results

# Solving the mathematical model via PULP 
results.Solving_Model(prob)

print("\n************* END: Solving mathematical model  *************\n")

######################################################################
#========================= SUMMARY RESULTS  ==========================
######################################################################

summary_results = SummaryResults(data, results) # Criando um objeto da classe SummaryResults

print("\n************* START: Summary Results  *************\n")

# The status of the solution is printed
print("Status: " + results.Status)

if results.Status == "Infeasible": 
	print("\nWARNING!!! Infeasible problem! Please revise the input data file and make sure all data is consistent!\n")
	print("Solution written to output_file.txt")
	print("\n************* END: Summary Results  *************\n")

	# Writing the summary results in an output file.txt
	summary_results.WritingUnfeasibleOutputFile()
	sys.exit()
else:
	# Show the value of the objective function
	print("\n======= Costs ======\n")
	print("Value of the objective function: " + "\t" + str(round(results.ObjectiveFunctionValue,0)) + "\t" + "R$")
	print("Investment cost: " + "\t" + "\t" + "\t" + str(round(sum([data.cESS[b] * (0.5 * results.MxEESS_beta[b] + 0.5 * results.MxPESS_beta[b]) for b in data.B]),0)) + "\t" + "R$")
	print("Operation cost: " + "\t" + "\t" + "\t" + str(round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cEDS[t] * data.deltat * results.PEDS[t][s][c] for (t,s,c) in List_TSC]),0)) + "\t" + "R$")
	print("Thermal generation cost: " + "\t" + "\t"  + str(round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cOTer[t] * data.deltat * results.PTER[t][s][c] for (t,s,c) in List_TSC]),0)) + "\t" + "R$")
	print("Load shedding cost: " + "\t" + "\t" + str(round(sum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.M * data.fD[t][s] * data.MxPD * results.xD[t][s][c] * data.deltat for (t,s,c) in List_TSC]),0)) + "\t" + "R$")

	# Show results
	print("\n====== Sizing ======\n")
	print("Thermal generation size (MxPTer): " + "\t" + str(round(results.MxPTer,0)) + "\t" + "kW")
	print("PV generation size (MxPPV): " + "\t" + "\t" + str(round(results.MxPPV,0)) + "\t" + "kW")
	print("ESS power converter (MxPESS): " + "\t" + str(round(results.MxPESS,0)) + "\t" + "kW")
	print("ESS energy capacity (MxEESS): " + "\t" + str(round(results.MxEESS,0)) + "\t" + "kWh")
	for a in data.A:
		if results.alpha[a] != 0: 
			print("Type of PV technology chosen:" + "\t" + str(data.PV_tech_name[a]))
	for b in data.B:
		if results.beta[b] != 0: 
			print("Type of ESS technology chosen:" + "\t" + str(data.ESS_tech_name[b]))

	# Writing the summary results in an output file.txt
	summary_results.WritingFeasibleOutputFile(List_TSC, List_TS, List_TSB, List_ATSC)

	print("\n************* END: Summary Results  *************\n")

	print("Solution written to output_file.txt")

""""
========================== Storage Results =========================
"""

database.storage(data, results, List_TSC)  # Storage result data in mongodb database

""""
========================== Plotting Results =========================
"""
figures = PlottingResults(data, results) # Creating an objetc of the class PlottingResults

# Creating the directory for the folders with the figures
script_dir = os.path.dirname(os.path.abspath(__file__))

print("\n************* START: Plotting final results  *************\n", flush=True)
print("Creating Figures...", flush=True)

# Creating Figures with the results 
figures.CreatingFigures()

print("\n************* END: Plotting final results  *************\n", flush=True)
print("Figures saved in: " + script_dir + "\Figures\n")
