""""
========================== Mathematical Modelling =========================
"""
import os
from pulp import *

class MathematicalModel: 
	def __init__(self, data):
		self.data = data

	def ConstructionOfLists(self): 
		# List formed by sets T, S and C
		data = self.data
		List_TSC = [];
		for t in data.T:
			for s in data.S:
				for c in data.C:
					List_TSC += [[t, s, c]]
		List_TS = [];
		for t in data.T:
			for s in data.S:
				List_TS += [[t, s]]
		List_SC = [];
		for s in data.S:
			for c in data.C:
				List_SC += [[s, c]]
		List_TSCB = [];
		for t in data.T:
			for s in data.S:
				for c in data.C:
					for b in data.B:
						List_TSCB += [[t, s, c, b]]
		List_ATSC = [];
		for a in data.A:
			for t in data.T:
				for s in data.S:
					for c in data.C:
						List_ATSC += [[a, t, s, c]]
		return (List_TSC, List_TS, List_SC, List_TSCB, List_ATSC)

	def ProblemFormulation(self, List_TSC, List_TS, List_SC, List_TSCB, List_ATSC):
		data = self.data
		# Type of problem
		prob = LpProblem("Optimal_Microgrid_Size",LpMinimize)

		# Declare variables
		MxPTer = LpVariable("MxPTer",0,data.limit_MxPTer)
		MxPPV = LpVariable("MxPPV",0,data.limit_MxPPV)
		MxPESS = LpVariable("MxPESS",0,data.limit_MxPESS)
		MxEESS = LpVariable("MxEESS",0,data.limit_MxEESS)
		MxPPV_alpha = LpVariable.dicts("MxPPV_alpha",data.A,0)
		MxPPV_alpha_xPV = LpVariable.dicts("MxPPV_alpha_xPV",(data.A,data.T,data.S,data.C),0)
		alpha = LpVariable.dicts("alpha",data.A,0,1,LpInteger)
		MxEESS_beta = LpVariable.dicts("MxEESS_beta",data.B,0)
		MxPESS_beta = LpVariable.dicts("MxPESS_beta",data.B,0)
		beta = LpVariable.dicts("beta",data.B,0,1,LpInteger)
		PESS_in = LpVariable.dicts("PESS_in",(data.T, data.S, data.C), 0)
		PESS_in_beta = LpVariable.dicts("PESS_in_beta",(data.T, data.S, data.C, data.B), 0)
		PESS_out = LpVariable.dicts("PESS_out",(data.T, data.S, data.C),0)
		PESS_out_beta = LpVariable.dicts("PESS_out_beta",(data.T, data.S, data.C, data.B), 0)
		PEDS = LpVariable.dicts("PEDS",(data.T, data.S, data.C))
		PTER = LpVariable.dicts("PTER",(data.T, data.S, data.C),0)
		xPV = LpVariable.dicts("xPV",(data.T, data.S, data.C),cat='Binary')
		xD = LpVariable.dicts("xD",(data.T, data.S, data.C),cat='Binary')
		xD_p = LpVariable.dicts("xD_p",(data.T, data.S, data.C),0,1)
		xD_n = LpVariable.dicts("xD_n",(data.T, data.S, data.C),0,1)
		gamma_in = LpVariable.dicts("gamma_in",(data.T, data.S, data.C),cat='Binary')
		gamma_out = LpVariable.dicts("gamma_out",(data.T, data.S, data.C),cat='Binary')
		EESS = LpVariable.dicts("EESS",(data.T, data.S, data.C),0)

		# Objective function
		prob += \
		   lpSum([data.cPV[a] * MxPPV_alpha[a] for a in data.A]) + lpSum([data.cESS[b] * (0.4 * MxEESS_beta[b] + 0.6 * MxPESS_beta[b]) for b in data.B]) + data.cITer * MxPTer + \
		   lpSum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cEDS[t] * data.deltat * PEDS[t][s][c] for (t,s,c) in List_TSC]) + \
		   lpSum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.p[s][c] * data.cOTer[t] * data.deltat * PTER[t][s][c] for (t,s,c) in List_TSC]) + \
		   lpSum([(1/((1.0 + data.r)**(data.d)) * ((1.0 + data.r)**(data.timeOp) - 1)/(data.r * (1.0 + data.r)**(data.timeOp))) * 365 * data.M * data.p[s][c] * data.fD[t][s] * data.MxPD * xD[t][s][c] * data.deltat  for (t,s,c) in List_TSC]), "Objective_Function"

		# Constraints
		for (t,s,c) in List_TSC:
		   prob += PEDS[t][s][c] + PTER[t][s][c] + lpSum([data.fPV[t][s] * data.eta[a] * MxPPV_alpha_xPV[a][t][s][c] for a in data.A]) - data.fD[t][s] * data.MxPD * (1 - xD[t][s][c]) - PESS_in[t][s][c] + PESS_out[t][s][c] == 0, "Power_Balance_Equation_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PEDS[t][s][c] <= data.MxPEDS, "PEDS_upper_limits_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PEDS[t][s][c] >= -data.MxPEDS, "PEDS_lower_limits_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PTER[t][s][c] <= MxPTer, "PTER_upper_limits_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_in[t][s][c] <= MxPESS, "PESS_in_upper_limits_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_out[t][s][c] <= MxPESS, "PESS_out_upper_limits_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_in[t][s][c] <= data.limit_MxPESS * gamma_in[t][s][c], "PESS_in_gamma_in_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_out[t][s][c] <= data.limit_MxPESS * gamma_out[t][s][c], "PESS_out_gamma_out_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += gamma_in[t][s][c] + gamma_out[t][s][c] <=  1, "Operation_ESS_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_in[t][s][c] <= data.limit_MxPESS * (1 - xD[t][s][c]), "PESS_loadshedding_limits1_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += PESS_out[t][s][c] <= data.limit_MxPESS * (1 - xD[t][s][c]), "PESS_loadshedding_limits2_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) > 1.0:
			   prob += EESS[t][s][c] - EESS[str(int(t)-1)][s][c] - data.deltat * lpSum([data.mu*data.eta_ESS[b]*PESS_in_beta[t][s][c][b] - 1/(data.mu*data.eta_ESS[b])*PESS_out_beta[t][s][c][b] for b in data.B]) + data.beta * data.deltat * EESS[t][s][c] == 0, "Initial_energy_condition_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) == 1.0:
			   prob += EESS[t][s][c] - MxEESS*0.5 - data.deltat * lpSum([data.mu*data.eta_ESS[b]*PESS_in_beta[t][s][c][b] - 1/(data.mu*data.eta_ESS[b])*PESS_out_beta[t][s][c][b] for b in data.B]) + data.beta * data.deltat * EESS[t][s][c] == 0, "Initial_energy_condition_2_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) > 1.0:
			   prob += EESS[t][s][c] - EESS[str(int(t)-1)][s][c] <= lpSum([data.ramp[b] * MxEESS_beta[b] for b in data.B]), "ramp_limits_1_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) > 1.0:
			   prob += EESS[t][s][c] - EESS[str(int(t)-1)][s][c] >= -lpSum([data.ramp[b] * MxEESS_beta[b] for b in data.B]), "ramp_limits_2_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) == 1.0:
			   prob += EESS[t][s][c] - MxEESS*0.5 <= lpSum([data.ramp[b] * MxEESS_beta[b] for b in data.B]), "ramp_limits_3_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) == 1.0:
			   prob += EESS[t][s][c] - MxEESS*0.5 >= -lpSum([data.ramp[b] * MxEESS_beta[b] for b in data.B]), "ramp_limits_4_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   prob += EESS[t][s][c] <= MxEESS, "Storage_limits_1_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) < int(c) or int(c) == 0.0:
			   prob += EESS[t][s][c] >= MxEESS * 0.5, "Storage_limits_2_%s" %str((t,s,c))

		prob += MxPESS >= MxEESS * 0.5, "EESS_PESS_limits_1_%s"

		prob += MxPESS <= MxEESS, "EESS_PESS_limits_2_%s"

		for (t,s,c) in List_TSC:
		   if int(t) < int(c):
			   prob += EESS[t][s][c] == EESS[t][s]['0'], "Storage_limits_3_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(t) >= int(c) and int(t) <= min(len(data.T),int(c)+(data.D/data.deltat)-1.0) and int(c) > 0:
			   prob += PEDS[t][s][c] == 0, "Contingency_constraint_1_%s" %str((t,s,c))

		for (t,s,c) in List_TSC:
		   if int(c) == 0:
			   prob += xD[t][s][c] == 0, "Contingency_constraint_2_%s" %str((t,s,c))

		for a in data.A:
			prob += MxPPV_alpha[a] <= data.limit_MxPPV * alpha[a], "Linearization_MxPPV_alpha_1_%s" %str(a)

		for a in data.A:
			prob += MxPPV_alpha[a] - MxPPV <= data.limit_MxPPV * (1 - alpha[a]), "Linearization_MxPPV_alpha_2_%s" %str(a)

		for a in data.A:
			prob += -data.limit_MxPPV * (1 - alpha[a]) <= MxPPV_alpha[a] - MxPPV, "Linearization_MxPPV_alpha_3_%s" %str(a)

		for (a,t,s,c) in List_ATSC:
			prob += MxPPV_alpha_xPV[a][t][s][c] <= data.limit_MxPPV * (1 - xPV[t][s][c]), "Linearization_MxPPV_alpha_xPV_1_%s" %str((a,t,s,c))

		for (a,t,s,c) in List_ATSC:
			prob += MxPPV_alpha_xPV[a][t][s][c] - MxPPV_alpha[a] <= data.limit_MxPPV * xPV[t][s][c], "Linearization_MxPPV_alpha_xPV_2_%s" %str((a,t,s,c))

		for (a,t,s,c) in List_ATSC:
			prob += -data.limit_MxPPV * xPV[t][s][c] <= MxPPV_alpha_xPV[a][t][s][c] - MxPPV_alpha[a], "Linearization_MxPPV_xPV_alpha_3_%s" %str((a,t,s,c))

		prob += lpSum([alpha[a] for a in data.A]) - 1 == 0, "Type_of_PV"

		for b in data.B:
			prob += MxEESS_beta[b] <= data.limit_MxEESS * beta[b], "Linearization_MxEESS_beta_1_%s" %str(b)

		for b in data.B:
			prob += MxEESS_beta[b] - MxEESS <= data.limit_MxEESS * (1 - beta[b]), "Linearization_MxEESS_beta_2_%s" %str(b)

		for b in data.B:
			prob += -data.limit_MxEESS * (1 - beta[b]) <= MxEESS_beta[b] - MxEESS, "Linearization_MxEESS_beta_3_%s" %str(b)

		for b in data.B:
			prob += MxPESS_beta[b] <= data.limit_MxPESS * beta[b], "Linearization_MxPESS_beta_1_%s" %str(b)

		for b in data.B:
			prob += MxPESS_beta[b] - MxPESS <= data.limit_MxPESS * (1 - beta[b]), "Linearization_MxPESS_beta_2_%s" %str(b)

		for b in data.B:
			prob += -data.limit_MxPESS * (1 - beta[b]) <= MxPESS_beta[b] - MxPESS, "Linearization_MxPESS_beta_3_%s" %str(b)

		for (t,s,c,b) in List_TSCB:
			prob += PESS_in_beta[t][s][c][b] <= data.limit_MxPESS * beta[b], "Linearization_PESS_in_beta_1_%s" %str((t,s,c,b))

		for (t,s,c,b) in List_TSCB:
			prob += PESS_in_beta[t][s][c][b] - PESS_in[t][s][c] <= data.limit_MxPESS * (1 - beta[b]), "Linearization_PESS_in_beta_3_%s" %str((t,s,c,b))

		for (t,s,c,b) in List_TSCB:
			prob += -data.limit_MxPESS * (1 - beta[b]) <= PESS_in_beta[t][s][c][b] - PESS_in[t][s][c], "Linearization_PESS_in_beta_4_%s" %str((t,s,c,b))

		for (t,s,c,b) in List_TSCB:
			prob += PESS_out_beta[t][s][c][b] <= data.limit_MxPESS * beta[b], "Linearization_PESS_out_beta_1_%s" %str((t,s,c,b))

		for (t,s,c,b) in List_TSCB:
			prob += PESS_out_beta[t][s][c][b] - PESS_out[t][s][c] <= data.limit_MxPESS * (1 - beta[b]), "Linearization_PESS_out_beta_3_%s" %str((t,s,c,b))

		for (t,s,c,b) in List_TSCB:
			prob += -data.limit_MxPESS * (1 - beta[b]) <= PESS_out_beta[t][s][c][b] - PESS_out[t][s][c], "Linearization_PESS_out_beta_4_%s" %str((t,s,c,b))

		prob += lpSum([beta[b] for b in data.B]) - 1 == 0, "Type_of_ESS"

		prob += lpSum([data.cPV[a] * MxPPV_alpha[a] for a in data.A]) + lpSum([data.cESS[b] * (0.4 * MxEESS_beta[b] + 0.6 * MxPESS_beta[b]) for b in data.B]) + data.cITer * MxPTer >= data.INV_LowerLimit, "Investemnt_Lower_Limit"

		prob += lpSum([data.cPV[a] * MxPPV_alpha[a] for a in data.A]) + lpSum([data.cESS[b] * (0.4 * MxEESS_beta[b] + 0.6 * MxPESS_beta[b]) for b in data.B]) + data.cITer * MxPTer <= data.INV_UpperLimit, "Investemnt_Upper_Limit"

		return prob, MxPTer, MxPPV, MxPESS, MxEESS, alpha, beta


	def TestingEspecificSolution(self, prob, SMxPTer, SMxPPV, SMxPESS, SMxEESS,S_alpha, S_beta, MxPTer, MxPPV, MxPESS, MxEESS, alpha, beta):
		prob += MxPTer == SMxPTer, "Fix_MxPTer"
		prob += MxPPV == SMxPPV, "Fix_MxPPV"
		prob += MxPESS == SMxPESS, "Fix_MxPESS"
		prob += MxEESS == SMxEESS, "Fix_MxEESS"
		prob += alpha[str(S_alpha)] == 1, "Fix_alpha['4']"
		prob += beta[str(S_beta)] == 1, "Fix_beta['2']"
		return prob


	def WritingProblemFile(self, prob, filename):
		# The problem data is written to an .lp file
		prob.writeLP(filename + ".lp")


	def PrintProblemScreen(self, prob):
		##Print problem
		print(prob)


	# Function to solve the model and obtain the results 
	def Solving_Model(self,prob):
		data = self.data
		try:
			prob.solve()
			#solver = GUROBI_CMD()
			#prob.solve(solver=solver)
		except Exception as e:
			cwd = os.getcwd()
			solverdir = 'cbc.exe' # extracted and renamed CBC solver binary
			solverdir = os.path.join(cwd, solverdir)
			solver = COIN_CMD(path=solverdir)
			prob.solve(solver)

		self.Status = LpStatus[prob.status]
		self.ObjectiveFunctionValue = value(prob.objective)

		Variablenames = prob.variables() # This is a list 

		# Getting a lists with the name and value of all problem variables 
		varDic = {}
		for v in prob.variables():
			varDic[v.name] = v.varValue
		self.varDic = varDic

		# Creating dictionaries for variables depending of data.A
		self.MxPPV_alpha = self.CreateDictionaryForEachVariable_A(varDic, 'MxPPV_alpha')
		self.alpha = self.CreateDictionaryForEachVariable_A(varDic, 'alpha')

		# Creating dictionaries for variables depending of data.B
		self.MxEESS_beta = self.CreateDictionaryForEachVariable_B(varDic, 'MxEESS_beta')
		self.MxPESS_beta = self.CreateDictionaryForEachVariable_B(varDic, 'MxPESS_beta')
		self.beta = self.CreateDictionaryForEachVariable_B(varDic, 'beta')

		# Creating dictionaries for variables depending of data.ATSC
		self.MxPPV_alpha_xPV = self.CreateDictionaryForEachVariable_ATSC(varDic, 'MxPPV_alpha_xPV')

		# Creating dictionaries for variables depending of data.TSC
		self.PESS_in = self.CreateDictionaryForEachVariable_TSC(varDic, 'PESS_in')
		self.PESS_out = self.CreateDictionaryForEachVariable_TSC(varDic, 'PESS_out')
		self.PEDS = self.CreateDictionaryForEachVariable_TSC(varDic, 'PEDS')
		self.PTER = self.CreateDictionaryForEachVariable_TSC(varDic, 'PTER')
		self.xPV = self.CreateDictionaryForEachVariable_TSC(varDic, 'xPV')
		self.xD = self.CreateDictionaryForEachVariable_TSC(varDic, 'xD')
		self.gamma_in = self.CreateDictionaryForEachVariable_TSC(varDic, 'gamma_in')
		self.gamma_out = self.CreateDictionaryForEachVariable_TSC(varDic, 'gamma_out')
		self.EESS = self.CreateDictionaryForEachVariable_TSC(varDic, 'EESS')

		# Creating dictionaries for variables non depending of data sets 
		self.MxPTer = self.CreateUniqueVariables(varDic, 'MxPTer')
		self.MxPPV = self.CreateUniqueVariables(varDic, 'MxPPV')
		self.MxPESS = self.CreateUniqueVariables(varDic, 'MxPESS')
		self.MxEESS = self.CreateUniqueVariables(varDic, 'MxEESS')


	def CreateDictionaryForEachVariable_TSC(self, varDic, variable_name):
		data = self.data
		aux_variable_name = variable_name + '_'
		variable_name = {}
		for t in data.T:
			variable_name[t] ={}
			for s in data.S:
				variable_name[t][s] ={}
				for c in data.C:
					for key in varDic:
						if key.startswith(aux_variable_name + t +'_'+ s +'_'+ c):
							variable_name[t][s][c] = varDic[key]
		return variable_name


	def CreateDictionaryForEachVariable_ATSC(self, varDic, variable_name):
		data = self.data
		aux_variable_name = variable_name + '_'
		variable_name = {}
		for a in data.A:
			variable_name[a] ={}
			for t in data.T:
				variable_name[a][t] ={}
				for s in data.S:
					variable_name[a][t][s] ={}
					for c in data.C:
						for key in varDic:
							if key.startswith(aux_variable_name + a +'_' + t +'_'+ s +'_'+ c):
								variable_name[a][t][s][c] = varDic[key]
		return variable_name


	def CreateDictionaryForEachVariable_A(self, varDic, variable_name):
		data = self.data
		aux_variable_name = variable_name + '_'
		variable_name = {}
		for a in data.A:
			for key in varDic:
				if key.startswith(aux_variable_name + a):
					variable_name[a] = varDic[key]
		return variable_name


	def CreateDictionaryForEachVariable_B(self, varDic, variable_name):
		data = self.data
		aux_variable_name = variable_name + '_'
		variable_name = {}
		for b in data.B:
			for key in varDic:
				if key.startswith(aux_variable_name + b):
					variable_name[b] = varDic[key]
		return variable_name


	def CreateUniqueVariables(self, varDic, variable_name):
		data = self.data
		aux_variable_name = variable_name
		for key in varDic:
			if key == aux_variable_name:
				variable_name = varDic[key]
		return variable_name
